package sk.codeblog.seccam;

import android.content.Context;
import android.graphics.ImageFormat;
import android.hardware.camera2.CameraAccessException;
import android.hardware.camera2.CameraCaptureSession;
import android.hardware.camera2.CameraCharacteristics;
import android.hardware.camera2.CameraDevice;
import android.hardware.camera2.CameraManager;
import android.hardware.camera2.CaptureRequest;
import android.hardware.camera2.params.StreamConfigurationMap;
import android.media.Image;
import android.media.ImageReader;
import android.os.Environment;
import android.util.Log;
import android.util.Size;

import androidx.annotation.NonNull;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import static android.content.Context.CAMERA_SERVICE;

public class Actions {

    private static final String TAG = "Actions";

    private static Context _context = null;
    private static CameraDevice _camera;
    private static CameraCaptureSession _capturesession;
    private static ImageReader _imgreader;
    private static String _cameraid = "";
    private static Size _camerasize = null;

    public static void Init(Context cntx) {

        //Ak uz bolo inicializovane nepokracujeme
        if (_context != null)
            return;

        _context = cntx;

        InitCamera();
    }

    public static void TakePhoto() {
        //Ak nebola najdena vhodna kamera koncime
        if (_cameraid.equals("") || _camerasize == null) {
            return;
        }

        //Ak existuje referencia na objekt kamery, pravdepodobne este nedobehla predchadzajuca akcia
        if (_camera != null) {
            return;
        }

        try {
            CameraManager manager = (CameraManager) _context.getSystemService(CAMERA_SERVICE);
            manager.openCamera(_cameraid, cameraStateCallback, null);

            _imgreader = ImageReader.newInstance(_camerasize.getWidth(), _camerasize.getHeight(), ImageFormat.JPEG, 1);
            _imgreader.setOnImageAvailableListener(onImageAvailableListener, null);

        } catch (CameraAccessException camEx) {
            Log.e(TAG, camEx.getMessage());
        } catch (SecurityException secEx) {
            Log.e(TAG, secEx.getMessage());
        } catch (Exception ex) {
            Log.e(TAG, ex.getMessage());
        }
    }

    private static void SendPhoto(String file) {
        GMailer th = new GMailer(file);
        th.start();
    }

    private static void InitCamera() {
        try {
            CameraManager manager = (CameraManager) _context.getSystemService(CAMERA_SERVICE);

            //Ziskame zoznam kamier
            String[] Cameras = manager.getCameraIdList();
            for (String CameraId : Cameras) {

                //Ziskame informacie o konkretnej kamere
                CameraCharacteristics CameraInfo = manager.getCameraCharacteristics(CameraId);
                //Orientacia kamery
                int LensFacing = CameraInfo.get(CameraCharacteristics.LENS_FACING);

                //Hladame zadnu kameru
                if (LensFacing == CameraCharacteristics.LENS_FACING_BACK) {

                    //Pokusime sa zistit najvacsie mozne rozlisenie snimkov
                    Size CameraSize = GetCameraResolution(CameraInfo);
                    if (CameraSize != null) {
                        _cameraid = CameraId;
                        _camerasize = CameraSize;
                        break;
                    }
                }
            }
        } catch (CameraAccessException camEx) {
            Log.e(TAG, camEx.getMessage());
        } catch (SecurityException secEx) {
            Log.e(TAG, secEx.getMessage());
        }
    }

    //Vyhladame najvacsie mozne rozlisenie fotografie
    private static Size GetCameraResolution(CameraCharacteristics CameraInfo) {
        StreamConfigurationMap streamConfigurationMap = CameraInfo.get(CameraCharacteristics.SCALER_STREAM_CONFIGURATION_MAP);
        Size[] Sizes = streamConfigurationMap.getOutputSizes(ImageFormat.JPEG);
        if (Sizes == null)
            return null;

        int Max = 0;
        Size MaxSize = null;

        for (Size CurrSize : Sizes) {
            int Width = CurrSize.getWidth();
            if (Width > Max) {
                Max = Width;
                MaxSize = CurrSize;
            }
        }
        return MaxSize;
    }

    private static CameraDevice.StateCallback cameraStateCallback = new CameraDevice.StateCallback() {
        @Override
        public void onOpened(@NonNull CameraDevice camera) {
            Log.d(TAG, "CameraDevice.StateCallback onOpened");
            try {
                _camera = camera;
                _camera.createCaptureSession(Arrays.asList(_imgreader.getSurface()), sessionStateCallback, null);
            } catch (CameraAccessException e) {
                Log.e(TAG, e.getMessage());
            }
        }

        @Override
        public void onDisconnected(@NonNull CameraDevice camera) {
            Log.w(TAG, "CameraDevice.StateCallback onDisconnected");
        }

        @Override
        public void onError(@NonNull CameraDevice camera, int error) {
            Log.e(TAG, "CameraDevice.StateCallback onError " + error);
        }
    };

    private static ImageReader.OnImageAvailableListener onImageAvailableListener = new ImageReader.OnImageAvailableListener() {
        @Override
        public void onImageAvailable(ImageReader reader) {
            Log.d(TAG, "onImageAvailable");
            Image img = reader.acquireLatestImage();
            if (img != null) {
                String FileName  = SaveImage(img);
                if (FileName != null)
                {
                    SendPhoto(FileName);
                }
                img.close();
            }

            //Uzavrieme kameru
            CloseCamera();
        }
    };

    private static void CloseCamera()
    {
        _camera.close();
        _camera = null;

        _capturesession.close();
        _capturesession = null;

        _imgreader.close();
        _imgreader = null;
    }

    private static CameraCaptureSession.StateCallback sessionStateCallback = new CameraCaptureSession.StateCallback() {

        @Override
        public void onReady(CameraCaptureSession session) {
            _capturesession = session;
            try {

                CaptureRequest.Builder builder = _camera.createCaptureRequest(CameraDevice.TEMPLATE_RECORD);
                builder.addTarget(_imgreader.getSurface());
                CaptureRequest Request = builder.build();

                session.capture(Request, null, null);
            } catch (CameraAccessException e) {
                Log.e(TAG, e.getMessage());
            }
        }

        @Override
        public void onConfigured(CameraCaptureSession session) {

        }

        @Override
        public void onConfigureFailed(@NonNull CameraCaptureSession session) {
        }
    };

    private static String GetFileName() {
        Date ActDate = new Date();
        GregorianCalendar Cal = new GregorianCalendar();
        Cal.setTime(ActDate);

        return String.format("%04d%02d%02d_%02d%02d%02d_%03d.jpg",
                Cal.get(Calendar.YEAR), Cal.get(Calendar.MONTH) + 1, Cal.get(Calendar.DAY_OF_MONTH),
                Cal.get(Calendar.HOUR_OF_DAY), Cal.get(Calendar.MINUTE), Cal.get(Calendar.SECOND),
                Cal.get(Calendar.MILLISECOND));
    }

    private static String SaveImage(Image image) {

        boolean IsOK = false;

        //Process image data
        ByteBuffer buffer;
        byte[] bytes;

        String FileName =  Environment.getExternalStorageDirectory() + "/Pictures/photo_" + GetFileName() + ".jpg";

        //getApplicationInfo().dataDir;
        File file = new File(FileName);
        FileOutputStream output = null;

        if (image.getFormat() == ImageFormat.JPEG) {
            buffer = image.getPlanes()[0].getBuffer();
            bytes = new byte[buffer.remaining()]; // makes byte array large enough to hold image
            buffer.get(bytes); // copies image from buffer to byte array
            try {
                output = new FileOutputStream(file);
                output.write(bytes);    // write the byte array to file

                IsOK = true;
            } catch (FileNotFoundException e) {
                Log.e(TAG, e.getMessage());
            } catch (IOException e) {
                Log.e(TAG, e.getMessage());
            } finally {
                image.close(); // close this to free up buffer for other images
                if (output != null) {
                    try {
                        output.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        }

        if (IsOK)
        {
            return FileName;
        }
        else
        {
            return null;
        }
    }
}
